"{ Package: 'stx:libscm/mercurial' }"

Object subclass:#HGCommandParser
	instanceVariableNames:'stream'
	classVariableNames:''
	poolDictionaries:''
	category:'SCM-Mercurial-Internal'
!


!HGCommandParser class methodsFor:'instance creation'!

on: aStream
    ^self new stream: aStream readStream

    "Created: / 23-10-2012 / 11:07:52 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 13-11-2012 / 11:02:08 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGCommandParser class methodsFor:'templates'!

templateLog
    ^
'{rev}:{node}
{branch}
{parents}
{author}
{date|isodate}
{desc}
**EOE**'

    "Created: / 12-11-2012 / 23:06:26 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 13-11-2012 / 10:21:55 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGCommandParser methodsFor:'accessing'!

stream
    ^ stream
!

stream:something
    stream := something.
! !

!HGCommandParser methodsFor:'error reporting'!

parseStatus
    | c |
    stream skipSeparators.
    c := stream peek.
    self halt.

    "Created: / 22-10-2012 / 21:43:03 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGCommandParser methodsFor:'parsing'!

parseDate
    self shouldImplement

    "Created: / 13-11-2012 / 10:22:46 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseLog
    "Parse output of 'hg log' command, assuming the template given
     was HGCommandParser templateLog. Return a list of HGRevision."

    | revs |

    revs := OrderedCollection new.
    [ stream atEnd ] whileFalse:[
        | rev |

        rev := self parseLogEntry.
        revs add: rev.
    ].

    ^revs.

    "Created: / 13-11-2012 / 09:46:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseLogEntry
    "Parse single revision entry, assuming the template given
     was HGCommandParser templateLog. Return a HGRevision."

    | rev line message |

    rev := HGRevision new.
    rev setId: self parseNodeId. self nextLineEnd.
    rev setBranch: self nextLine.
    rev setParent1Id: self parseNodeId. self nextSpace.
    rev setParent2Id: self parseNodeId. self nextLineEnd.
    rev setAuthor: self nextLine.
    rev setDate: self parseDate. self nextLineEnd.
    message := String streamContents:[:s|
        [ line := self nextLine . line = '**EOE**' ] whileFalse:[
            s nextPutLine: line
        ].
    ].
    rev setMessage: message

    "Created: / 13-11-2012 / 09:45:40 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseNodeId
    "Parses node id from stream and returns it. Support both,
     short and full node ids"

    | c c2 sign short revno hash |

    stream peek == $- ifTrue:[        
        stream next.
        sign := -1.
    ] ifFalse:[
        sign := 1.
    ].

    "/ Read revno...
    revno := 0.
    [ (c := stream peek) == $: ] whileFalse:[
        c isDigit ifFalse:[
            self error:'Digit ([0-9]) expected but ', c , 'found'.
        ].
        revno := (revno * 10) + c digitValue.
        stream next.
    ].
    stream next. "/eat :
    revno := revno * sign.

    "/ Read hash
    hash := ByteArray new: 20.
    short := true.
    1 to: 6 do:[:i|
        stream atEnd ifTrue:[
            self error:'Unexpected end of stream, hex digit expected'.
        ].
        c := stream peek.
        c isHexDigit ifFalse:[
            self error:'Hex digit ([0-9a-z]) expected but ', c , ' found'.
        ].
        stream next.
        stream atEnd ifTrue:[
            self error:'Unexpected end of stream, hex digit expected'.
        ].
        c2 := stream peek.
        c isHexDigit ifFalse:[
            self error:'Hex digit ([0-9a-z]) expected but ', c , ' found'.
        ].
        hash at:i put: (c digitValue << 4) + c2 digitValue.
        stream next.
    ].
    stream peek isHexDigit ifTrue:[
        "/OK, full 40-char node id
        short := false.
        7 to: 20 do:[:i|
            stream atEnd ifTrue:[
                self error:'Unexpected end of stream, hex digit expected'.
            ].
                        c := stream peek.
            c isHexDigit ifFalse:[
                self error:'Hex digit ([0-9a-z]) expected but ', c , ' found'.
            ].
            stream next.
            stream atEnd ifTrue:[
                self error:'Unexpected end of stream, hex digit expected'.
            ].
            c2 := stream peek.
            c isHexDigit ifFalse:[
                self error:'Hex digit ([0-9a-z]) expected but ', c , ' found'.
            ].
            hash at:i put: (c digitValue << 4) + c2 digitValue.
            stream next.
        ].
    ].
    ^short ifTrue:[
        self halt: 'Not yet supported'
    ] ifFalse:[
        (HGNodeId fromBytes: hash) revno: revno.
    ]

    "
        (HGCommandParser on: '4:6f88e1f44d9eb86e0b56ca15e30e5d786acd83c7' readStream) parseNodeId

        Bad ones:

        (HGCommandParser on: '4:6f88e1f44d9eb86e0b56ca15e30e5d786acd' readStream) parseNodeId
        (HGCommandParser on: '4:6f88Z1f44d9eb86e0b56ca15e30e5d786acd83c7' readStream) parseNodeId

    "

    "Created: / 13-11-2012 / 10:22:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGCommandParser methodsFor:'parsing - commands'!

parseCommandLog
    "Parse output of 'hg log' command, assuming the template given
     was HGCommandParser templateLog. Return a list of HGRevision."

    ^self parseLog

    "Created: / 13-11-2012 / 09:09:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

parseCommandStatus
    | statusesAndPaths |
    statusesAndPaths := OrderedCollection new.
    [ stream atEnd ] whileFalse:[
        | status path |

        status := HGStatus forCode: self next.
        self nextSpace.
        path := self nextLine.
        statusesAndPaths add: { status . path }
    ].
    ^ statusesAndPaths

    "Created: / 23-10-2012 / 10:57:06 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGCommandParser methodsFor:'parsing-utils'!

next
    ^stream next.

    "Created: / 23-10-2012 / 10:57:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

nextLine
    ^stream nextLine

    "Created: / 23-10-2012 / 11:05:47 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 09-11-2012 / 12:02:08 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

nextLineEnd
    | c |
    ((c := stream next) ~= Character cr) ifTrue:[
        self error:'New line expected. ''', c , ''' found!!'
    ]

    "Created: / 13-11-2012 / 10:17:28 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

nextSpace
    | c |
    ((c := stream next) ~= Character space) ifTrue:[
        self error:'Space expected. ''', c , ''' found!!'
    ]

    "Created: / 23-10-2012 / 10:58:20 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 09-11-2012 / 11:59:45 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!HGCommandParser class methodsFor:'documentation'!

version_SVN
    ^ '$Id::                                                                                                                        $'
! !
